from datetime import datetime, timedelta

from pyramid.view import view_config
from sqlalchemy import func

from barker.models import Inventory, Kot, Product, Voucher, VoucherType
from barker.models.validation_exception import ValidationError
from barker.views.reports import get_start_date, get_finish_date


@view_config(
    request_method="GET",
    route_name="v1_beer_consumption_report",
    renderer="json",
    permission="Beer Consumption",
)
def beer_consumption(request):
    start_date = get_start_date(request.GET.get("s", None)) - timedelta(hours=7)
    finish_date = get_finish_date(request.GET.get("f", None)) - timedelta(hours=7)

    if (
        datetime.today() - start_date.replace(hour=0)
    ).days > 5 and "Accounts Audit" not in request.effective_principals:
        raise ValidationError("Accounts Audit")

    day = func.date_trunc("day", Voucher.date - timedelta(hours=7)).label("day")
    sum_ = func.sum(Inventory.quantity * Product.quantity).label("sum")
    list_ = (
        request.dbsession.query(day, Product.name, sum_)
        .join(Voucher.kots)
        .join(Kot.inventories)
        .join(Inventory.product)
        .filter(
            day >= start_date,
            day <= finish_date,
            Voucher.voucher_type.in_(
                [
                    VoucherType.REGULAR_BILL.value,
                    VoucherType.NO_CHARGE.value,
                    VoucherType.STAFF.value,
                ]
            ),
        )
        .group_by(day, Product.name)
        .having(sum_ != 0)
        .order_by(day, Product.name)
        .all()
    )
    headers = []
    data = []
    for date, name, quantity in list_:
        if name not in headers:
            headers.append(name)
        old = [d for d in data if d["date"] == date.strftime("%d-%b-%Y")]
        if len(old):
            old[0][name] = quantity
        else:
            data.append({"date": date.strftime("%d-%b-%Y"), name: quantity})
    return {
        "startDate": start_date.date().strftime("%d-%b-%Y"),
        "finishDate": (finish_date - timedelta(days=1)).date().strftime("%d-%b-%Y"),
        "headers": headers,
        "data": data,
    }
