from datetime import datetime, timedelta

from pyramid.view import view_config

from barker.models import Settlement, SettleOption, Voucher, VoucherType
from barker.models.validation_exception import ValidationError
from barker.views.reports import get_start_date, get_finish_date


@view_config(
    request_method="GET",
    route_name="v1_bill_settlement_report",
    renderer="json",
    permission="Bill Details",
)
def bill_details(request):
    start_date = get_start_date(request.GET.get("s", None))
    finish_date = get_finish_date(request.GET.get("f", None))

    if (
        datetime.today() - start_date.replace(hour=0)
    ).days > 5 and "Accounts Audit" not in request.effective_principals:
        raise ValidationError("Accounts Audit")

    vouchers = (
        request.dbsession.query(Voucher)
        .join(Voucher.settlements)
        .join(Settlement.settle_option)
        .filter(Voucher.date >= start_date, Voucher.date <= finish_date)
        .order_by(Voucher.voucher_type)
        .order_by(Voucher.bill_id)
        .all()
    )

    report = []
    for item in vouchers:
        if item.voucher_type == VoucherType.VOID:
            amount = (
                next(
                    s.amount
                    for s in item.settlements
                    if s.settled == SettleOption.AMOUNT()
                )
                * -1
            )
            report.append(
                {
                    "date": item.date.strftime("%d-%b-%Y %H:%M:%S"),
                    "billId": item.full_bill_id,
                    "settlement": "Void: {0}".format(item.void_reason),
                    "amount": round(amount, 2),
                }
            )
        else:
            for so in (
                so for so in item.settlements if so.settle_option.show_in_choices
            ):
                report.append(
                    {
                        "date": item.date.strftime("%d-%b-%Y %H:%M:%S"),
                        "billId": item.full_bill_id,
                        "settlement": so.settle_option.name,
                        "amount": round(so.amount, 2),
                    }
                )
    return {
        "startDate": start_date.date().strftime("%d-%b-%Y"),
        "finishDate": (finish_date - timedelta(days=1)).date().strftime("%d-%b-%Y"),
        "amounts": report,
    }
