from datetime import datetime, timedelta

from pyramid.view import view_config
from sqlalchemy import func

from barker.models import Inventory, Kot, Product, Voucher, SaleCategory, VoucherType
from barker.models.validation_exception import ValidationError
from barker.views.reports import get_start_date, get_finish_date


@view_config(
    request_method="GET",
    route_name="v1_discount_report",
    renderer="json",
    permission="Discount Report",
)
def discount_report_view(request):
    start_date = get_start_date(request.GET.get("s", None))
    finish_date = get_finish_date(request.GET.get("f", None))

    if (
        datetime.today() - start_date.replace(hour=0)
    ).days > 5 and "Accounts Audit" not in request.effective_principals:
        raise ValidationError("Accounts Audit")

    return {
        "startDate": start_date.date().strftime("%d-%b-%Y"),
        "finishDate": (finish_date - timedelta(days=1)).date().strftime("%d-%b-%Y"),
        "amounts": discount_report(start_date, finish_date, request.dbsession),
    }


def discount_report(start_date, finish_date, dbsession):
    amount = func.sum(
        Inventory.quantity * Inventory.effective_price * Inventory.discount
    ).label("Amount")
    list_ = (
        dbsession.query(SaleCategory.name, amount)
        .join(Voucher.kots)
        .join(Kot.inventories)
        .join(Inventory.product)
        .join(Product.sale_category)
        .filter(
            Inventory.discount != 0,
            Voucher.date >= start_date,
            Voucher.date <= finish_date,
            Voucher.voucher_type.in_(
                [VoucherType.REGULAR_BILL.value, VoucherType.KOT.value]
            ),
        )
        .group_by(SaleCategory.name)
        .order_by(SaleCategory.name)
        .all()
    )

    return [{"name": pg, "amount": amt} for pg, amt in list_]
