from datetime import datetime, timedelta

from pyramid.view import view_config

from barker.models import Settlement, SettleOption, Voucher, VoucherType, Reprint
from barker.models.validation_exception import ValidationError
from barker.views.reports import get_start_date, get_finish_date


@view_config(
    request_method="GET",
    route_name="v1_voids_reprints_report",
    renderer="json",
    permission="Void or Reprinted Bill Report",
)
def voids_reprints_report_view(request):
    start_date = get_start_date(request.GET.get("s", None))
    finish_date = get_finish_date(request.GET.get("f", None))

    if (
        datetime.today() - start_date.replace(hour=0)
    ).days > 5 and "Accounts Audit" not in request.effective_principals:
        raise ValidationError("Accounts Audit")

    return {
        "startDate": start_date.date().strftime("%d-%b-%Y"),
        "finishDate": (finish_date - timedelta(days=1)).date().strftime("%d-%b-%Y"),
        "amounts": voids_reprints_report(start_date, finish_date, request.dbsession),
    }


def voids_reprints_report(start_date, finish_date, dbsession):
    vouchers = (
        dbsession.query(Voucher, Settlement.amount * -1)
        .join(Voucher.settlements)
        .filter(Voucher.date >= start_date, Voucher.date <= finish_date, Voucher.voucher_type == VoucherType.VOID.value, Settlement.settled == SettleOption.AMOUNT())
        .order_by(Voucher.date)
        .all()
    )
    print(str(vouchers))
    report = []
    for item, amount in vouchers:
        report.append(
            {
                "date": item.date.strftime("%d-%b-%Y %H:%M:%S"),
                "billId": item.full_bill_id,
                "settlement": f"Void: {item.void_reason}",
                "amount": round(amount, 2),
            }
        )
    reprints = dbsession.query(Reprint).filter(
        Reprint.date >= start_date,
        Reprint.date <= finish_date
    ).all()

    for item in reprints:
        report.append({
            'date': item.date.strftime('%d-%b-%Y %H:%M:%S'),
            'billId': item.voucher.full_bill_id,
            'settlement': f"Reprinted by {item.user.name}",
            'Amount': round(next(
                s.amount for s in item.voucher.settlements if s.settled == SettleOption.AMOUNT()
            ) * -1, 2)
        })
    return report
