import { Injectable } from '@angular/core';
import { catchError } from 'rxjs/operators';
import { Observable } from 'rxjs/internal/Observable';
import { HttpClient, HttpHeaders, HttpParams } from '@angular/common/http';
import { TaxAnalysis } from './tax-analysis';
import { ErrorLoggerService } from '../core/error-logger.service';

const httpOptions = {
  headers: new HttpHeaders({'Content-Type': 'application/json'})
};

const url = '/v1/sale-analysis/tax';
const serviceName = 'TaxAnalysisService';

@Injectable({
  providedIn: 'root'
})
export class TaxAnalysisService {

  constructor(private http: HttpClient, private log: ErrorLoggerService) {
  }

  get(startDate: string, finishDate): Observable<TaxAnalysis> {
    const options = {params: new HttpParams()};
    if (startDate !== null) {
      options.params = options.params.set('s', startDate);
    }
    if (finishDate !== null) {
      options.params = options.params.set('f', finishDate);
    }
    return <Observable<TaxAnalysis>>this.http.get<TaxAnalysis>(url, options)
      .pipe(
        catchError(this.log.handleError(serviceName, 'get'))
      );
  }
}
